@@ -382,7 +382,19 @@ class FileData(_AbstractFileData):
             diffopts.git = True
             m = match.exact(repo.root, repo.root, [wfile])
             fp = cStringIO.StringIO()
-            for c in patch.diff(repo, ctx.node(), None, match=m, opts=diffopts):
+
+            try:
+                copy = {}
+                if oldname != wfile:
+                    copy[wfile] = oldname
+                patches = patch.diff(repo, ctx.node(), None, match=m,
+                                     opts=diffopts, copy=copy)
+            except TypeError:
+                # hg<3.9 (40d53d4b5925)
+                patches = patch.diff(repo, ctx.node(), None, match=m,
+                                     opts=diffopts)
+
+            for c in patches:
                 fp.write(c)
             fp.seek(0)
 
