@@ -68,6 +68,24 @@ def _querytype(repo, query):
         pass
     return 'keyword'
 
+
+class SelectAllLineEdit(QLineEdit):
+    def __init__(self, parent=None):
+        super(SelectAllLineEdit, self).__init__(parent)
+        self.just_got_focus = False
+
+    def focusInEvent(self, ev):
+        if ev.reason() == Qt.MouseFocusReason:
+            self.just_got_focus = True
+        super(SelectAllLineEdit, self).focusInEvent(ev)
+
+    def mouseReleaseEvent(self, ev):
+        if self.just_got_focus:
+            self.just_got_focus = False
+            self.selectAll()
+        super(SelectAllLineEdit, self).mouseReleaseEvent(ev)
+
+
 class RepoFilterBar(QToolBar):
     """Toolbar for RepoWidget to filter changesets"""
 
@@ -350,9 +368,16 @@ class RepoFilterBar(QToolBar):
         self._branchLabel.setMenu(self._branchMenu)
 
         self._branchCombo = QComboBox()
+        self._branchCombo.setEditable(True)
+        self._branchCombo.setInsertPolicy(QComboBox.NoInsert)
+        self._branchCombo.setLineEdit(SelectAllLineEdit())
+        self._branchCombo.lineEdit().editingFinished.connect(
+            self._lineBranchChanged)
         self._branchCombo.setMinimumContentsLength(10)
         self._branchCombo.setMaxVisibleItems(30)
         self._branchCombo.currentIndexChanged.connect(self._emitBranchChanged)
+        completer = QCompleter(self._branchCombo.model(), self._branchCombo)
+        self._branchCombo.setCompleter(completer)
 
         self.addWidget(self._branchLabel)
         self.addWidget(qtlib.Spacer(2, 2))
@@ -395,38 +420,35 @@ class RepoFilterBar(QToolBar):
         priomap = {self._repo.dirstate.branch(): -2, 'default': -1}
         branches = sorted(branches, key=lambda e: priomap.get(e, 0))
 
+        branches = map(hglib.tounicode, branches)
+
         self._branchCombo.blockSignals(True)
         self._branchCombo.clear()
         self._branchCombo.addItem(self._allBranchesLabel)
-        for branch in branches:
-            self._branchCombo.addItem(hglib.tounicode(branch))
-            self._branchCombo.setItemData(self._branchCombo.count() - 1,
-                                          hglib.tounicode(branch),
-                                          Qt.ToolTipRole)
+        self._branchCombo.setItemData(self._branchCombo.count() - 1, '')
+        for i, branch in enumerate(branches, self._branchCombo.count()):
+            self._branchCombo.addItem(branch)
+            self._branchCombo.setItemData(i, branch, Qt.ToolTipRole)
+            self._branchCombo.setItemData(i, branch)
         self._branchCombo.setEnabled(self.filterEnabled and bool(branches))
+        self.setBranch(curbranch)
         self._branchCombo.blockSignals(False)
 
-        if curbranch and curbranch not in branches:
+        if self.branch() != curbranch:
             self._emitBranchChanged()  # falls back to "show all"
-        else:
-            self.setBranch(curbranch)
 
     @pyqtSlot(str)
     def setBranch(self, branch):
         """Change the current branch by name [unicode]"""
-        if not branch:
-            index = 0
-        else:
-            index = self._branchCombo.findText(branch)
+        index = self._branchCombo.findData(branch)
         if index >= 0:
             self._branchCombo.setCurrentIndex(index)
 
     def branch(self):
         """Return the current branch name [unicode]"""
-        if self._branchCombo.currentIndex() == 0:
-            return ''
-        else:
-            return unicode(self._branchCombo.currentText())
+        index = self._branchCombo.currentIndex()
+        branch = self._branchCombo.itemData(index).toString()
+        return unicode(branch)
 
     def branchAncestorsIncluded(self):
         return self._allparAction.isChecked()
@@ -442,6 +464,12 @@ class RepoFilterBar(QToolBar):
         self.branchChanged.emit(self.branch(),
                                 self.branchAncestorsIncluded())
 
+    @pyqtSlot()
+    def _lineBranchChanged(self):
+        written_branch = self._branchCombo.currentText()
+        if not written_branch:
+            self._branchCombo.setCurrentIndex(0)
+
     @pyqtSlot()
     def refresh(self):
         self._updateBranchFilter()
